/*
	NagWorld.cpp
	Copyright(C) HB Päron Data, SWEDEN
	This file may be used under the terms of the Nabber License.
*/


#ifndef NAG_WINDOW_H
#include "NagWindow.h"
#endif
#ifndef NABBER_H
#include "NagWorld.h"
#endif
#include <Roster.h>
#include <Alert.h>
#include <Messenger.h>
#include <TextView.h>
#include <String.h>
#include <malloc.h>
#include <stdlib.h>


//stuff
const char *app_signature = "application/x-vnd.hbparondata-Nabber";
const char *NabberConfigFile = "/boot/home/config/settings/nabber.config";
const char *NabberNaggerList = "/boot/home/config/settings/nagger.lst";
const char *strOperaBrowser = "suite/x-vnd.opera-browserwindow";
BString NaggerListContent;
BString fileContent;

//START!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
int main(int, char**)
{
	NagApplication nagApp;
	nagApp.Run();
	return(0);
}

/////////////////////////////////// APPLICATION-STUFF /////////////////////////////////////////
NagApplication::NagApplication()
		  		  : BApplication(app_signature)
{
	//read files
	ReadNaggerList();
	ReadSettingsFile();
	
	//start the nagger-thread, run it later on...
	fThread = spawn_thread((status_t (*)(void *))MyThread, "naggring_thread", 1, this);
	
	const char *knappCaption;
	knappCaption = "Start Naggering";
	int32 pos, pos_end, startcut, endcut;
	BString buffer;
	pos_end = 0;
	pos = fileContent.FindFirst("AlwaysNagger", pos_end);
	if(pos != B_ERROR) {
		pos_end = fileContent.FindFirst('\n',pos);
		if(pos_end != B_ERROR) {
			if(pos>0){
				//start naggering on program-start?
				startcut = pos + strlen("AlwaysNagger")+1;
				endcut = pos_end - pos - strlen("AlwaysNagger")-1;
				fileContent.CopyInto(buffer,startcut,endcut);
				if(strcmp(buffer.String(),"yes")==false) {
					resume_thread(fThread);
					knappCaption = "Stop Naggering";
				}
			}
		}
	}

	BRect aRect;
	aRect.Set(100, 80, 400, 200);
	aWindow = new NagWindow(aRect, knappCaption);
	aWindow->Show();
}

void
NagApplication::MessageReceived(BMessage *message)
{
	switch (message->what) {
		case 'stop':
		{	//stop thread
			suspend_thread(fThread);
		} break;
		case 'rsum':
		{	//resume thread
			resume_thread(fThread);
		} break;
		default:
			BApplication::MessageReceived(message);
			break;
	}
}

/////////////////////////////////// NAGGER-THREAD ///////////////////////////////////
status_t 
NagApplication::MyThread(NagApplication *nagApp)
{
	bool EndLoop;
	status_t result;
	EndLoop = false;
	app_info infApp;
	
	while(EndLoop==false) {
		result = snooze(1);
		//if(result == B_OK) {
			be_roster->GetActiveAppInfo(&infApp);
			nagApp->KillWindow(infApp);
		//}
	}
	return B_NO_ERROR;
}


status_t
NagApplication::KillWindow(app_info infApp)
{
	const char *namn;
	BRect frame;
	int32 wincount;
	BMessage reply;
	BList teams;
	status_t result;
	BMessage gMsg('nags');
	bool NaggerCorrect;
	NaggerCorrect = false;
	const char *strng;
	int32 pos2, pos2_end;
	BMessage winname_msg(B_GET_PROPERTY);
	BMessage winframe_msg(B_GET_PROPERTY);
	BMessage wnd_msg(B_GET_SUPPORTED_SUITES);
	BString buffer;
	int32 pos, pos_end, startcut, endcut;
	bool QuitLooping;
	
	pos = 0;
	pos_end = 0;
	while(pos!=B_ERROR) {
		pos = 0;
		//check every app defined in the variable "fileContent"
		pos = fileContent.FindFirst("NagApp",pos_end);
		if(pos != B_ERROR) {
			pos_end = fileContent.FindFirst('\n',pos);
			if(pos_end != B_ERROR) {
				if(pos>0){
					//copy app-signature into a variable
					BString sign;
					int32 stc = pos + 8;
					int32 edc = pos_end - pos - 8;
					fileContent.CopyInto(sign,stc,edc);
					//security - check so that we don't by accident nagger a app we don't want to nagger....
					if(strcmp(infApp.signature,sign.String()) == false) {
						BMessenger msgr(infApp.signature, infApp.team);
						//count windows
						BMessage count_msg(B_COUNT_PROPERTIES);
						count_msg.AddSpecifier("Window");
						result = msgr.SendMessage(&count_msg, &reply);
						if(result == B_OK){
							result = reply.FindInt32("result", 0, &wincount);
							if(result == B_OK){
								for(int32 wc=0; wc<(wincount+1); wc++){
									wnd_msg.MakeEmpty();
									wnd_msg.AddSpecifier("Window",wc);
									result = msgr.SendMessage(&wnd_msg, &reply);
									if(result == B_OK){
										result = reply.FindString("suites", 0, &strng);
										if(result == B_OK){
											//check if it's really a opera-browser window (if we don't do this nabber will kill all windows in opera (or any other app) that matches the nagger-conditions)
											if(strcmp(strng,strOperaBrowser)==false){
												winname_msg.MakeEmpty();
												winname_msg.AddSpecifier("Title");
												winname_msg.AddSpecifier("Window", wc);
												result = msgr.SendMessage(&winname_msg, &reply);
												if(result == B_OK){
													result = reply.FindString("result", 0, &namn);
													if(result==B_OK) {
														BString name(namn);
														//NAGGRINGS-START!!!!!!
														QuitLooping = false;
														while(QuitLooping == false){
															//find out size...
															winframe_msg.MakeEmpty();
															winframe_msg.AddSpecifier("Frame");
															winframe_msg.AddSpecifier("Window", wc);
															result = msgr.SendMessage(&winframe_msg, &reply);
															if(result == B_OK) {
																result = reply.FindRect("result", 0, &frame);
																if(result == B_OK) {
																	//nagger if the window matches the conditions
																	//NaggerListContent (BString) contans all condition-information
																	pos2 = 0;
																	pos2_end = 0;
																	//textcondition
																	while(pos2 != B_ERROR){
																		pos2 = NaggerListContent.FindFirst("TextCondition=",pos2_end);
																		if(pos2 != B_ERROR) {
																			pos2_end = NaggerListContent.FindFirst('\n',pos2);
																			if(pos2_end != B_ERROR) {
																				if(pos2>0){
																					//check so that it is the right condition (there is both TextCondition and SizeAndTextConditon)
																					BString kalle;
																					NaggerListContent.CopyInto(kalle,pos2-1,1);
																					if(strcmp(kalle.String(),"\n") == false) {
																						//copy condition into variable
																						startcut = pos2 + strlen("TextCondition=");
																						endcut = pos2_end - pos2 - strlen("TextCondition=");
																						NaggerListContent.CopyInto(buffer,startcut,endcut);
																						buffer.Truncate(buffer.CountChars()-1);
																						if(strcmp(buffer.String(),name.String()) == false) {
																							NaggerCorrect = CheckForExceptions(buffer.String(), frame, "från text1");
																						}
																						else {
																							if(strcmp(buffer.String(),"*") == false) {
																								NaggerCorrect = CheckForExceptions(name.String(), frame, "från text2");
																							}
																							else {
																								//window-title doesn't matck condition, check for wildcards
																								
																							} //(any window)
																						} //is cond=title
																					} //right cond.
																				} //pos2>0
																			} //b_error
																		} //b_error
																	} //while
																	pos2 = 0;
																	pos2_end = 0;
																	//size-condition
																	bool gg  = true;
																	if(!NaggerCorrect) {
																		BString del1;
																		BString del2;
																		while(pos2 != B_ERROR){
																			pos2 = NaggerListContent.FindFirst("SizeCondition=",pos2_end);
																			if(pos2 != B_ERROR) {
																				pos2_end = NaggerListContent.FindFirst('\n',pos2);
																				if(pos2_end != B_ERROR) {
																					if(pos2>0){
																						//copy cond into variable
																						startcut = pos2 + strlen("SizeCondition=");
																						endcut = pos2_end - pos2 - strlen("SizeCondition=");
																						NaggerListContent.CopyInto(buffer,startcut,endcut);
																						buffer.Truncate(buffer.CountChars()-1);
																						int32 kalle = buffer.FindFirst("	",0);
																						if(kalle != B_ERROR) {
																							//make 2 var. of 1
																							buffer.CopyInto(del1,0,kalle);
																							buffer.CopyInto(del2,kalle+1,buffer.CountChars()-kalle-1);
																							BString grej;
																							for(int qas=0;qas<2;qas++) {
																								if(qas==1)
																									grej.SetTo(del1);
																								else
																									grej.SetTo(del2);
																								kalle = grej.FindFirst("-",0);
																								int size;
																								if(qas==0)
																									size = (int)(frame.right - frame.left);
																								else
																									size = (int)(frame.bottom - frame.top);
																								if(kalle != B_ERROR) {
																									//is -
																									grej.CopyInto(buffer,0,kalle);
																										int sz1 = atoi(buffer.String());
																									grej.CopyInto(buffer,kalle+1,grej.CountChars()-kalle-1);
																										int sz2 = atoi(buffer.String());
																									if(size > sz1 && size < sz2 && gg) {
																										NaggerCorrect = CheckForExceptions(name.String(), frame, "från size1");
																										gg = true;
																									} else {
																										NaggerCorrect = false;
																										gg = false;
																									}
																								}
																								else {
																									//is no -
																									int sz = atoi(grej.String());
																									if(size == sz)
																										NaggerCorrect = CheckForExceptions(name.String(), frame, "från size2");
																									else
																										NaggerCorrect = false;
																								} //b_error
																							} //for
																						} //b_error
																					} //pos2>0
																				} //b_error
																			} //b_error
																		}  //while
																	}  //naggercorrect
																	
																} //b_ok
															} //b_ok
															//nagger or not? that's the question...
															if(NaggerCorrect) {
																//remove-window-message
																BMessage e_msg(B_QUIT_REQUESTED);
																e_msg.AddBool("shortcut",false);
																e_msg.AddSpecifier("Window",wc);
																//send mess to right app
																BMessenger emsgr(infApp.signature,infApp.team);
																emsgr.SendMessage(&e_msg, &reply);
																aWindow->PostMessage(&gMsg);
																NaggerCorrect = false;
															} //naggercorrect
															else {
																QuitLooping = true;
															}
														} //while
														QuitLooping = false;
														//NAGGRINGS-END!!!!!!
													} //b_ok
												} //b_ok
											} //if-opera
										} //b_ok
									} //b_ok
								} //for
							} //b_ok
						} //b_ok
					} //is app-siganature?
				}
			}
		}
	} //while
	return B_OK;
}

bool 
NagApplication::CheckForExceptions(const char *window_title, BRect frame, const char *varifran)
{
	//find exceptions
	int32 pos, pos_end, startcut, endcut;
	bool QuitLooping = false;
	bool IsExceptions = true; //if this one is true there is no exceptions (will remove window)
	pos = 0;
	pos_end = 0;
	startcut = 0;
	endcut = 0;
	BString buffer;
	buffer.SetTo("");
	//NaggerListContent contains the nagger-lust
	//check sizecond.
	BString del1;
	BString del2;
	while(QuitLooping == false){
		pos = NaggerListContent.FindFirst("SizeConditionException",pos_end);
		if(pos != B_ERROR) {
			pos_end = NaggerListContent.FindFirst('\n',pos);
			if(pos_end != B_ERROR) {
				if(pos>0){
					//copy into var.
					startcut = pos + strlen("SizeConditionException")+1;
					endcut = pos_end - pos - strlen("SizeConditionException");
					NaggerListContent.CopyInto(buffer,startcut,endcut);
					buffer.Truncate(buffer.CountChars()-2);
					int32 kalle = buffer.FindFirst("	",0);
					if(kalle != B_ERROR) {
						//copy into var.
						buffer.CopyInto(del1,0,kalle);
						buffer.CopyInto(del2,kalle+1,buffer.CountChars()-kalle-1);
						BString grej;
						bool gg = true;
						for(int qas=0;qas<2;qas++) {
							if(qas==1)
								grej.SetTo(del1);
							else
								grej.SetTo(del2);
							kalle = grej.FindFirst("-",0);
							int size;
							if(qas==0)
								size = (int)(frame.right - frame.left);
							else
								size = (int)(frame.bottom - frame.top);
							if(kalle != B_ERROR) {
								//is -
								grej.CopyInto(buffer,0,kalle);
									int sz1 = atoi(buffer.String());
								grej.CopyInto(buffer,kalle+1,grej.CountChars()-kalle-1);
									int sz2 = atoi(buffer.String());
								if(size > sz1 && size < sz2 && gg) {
									IsExceptions = false; //don't kill wnd if excpeption match
									gg  = true;
									QuitLooping = true;
								} else {
									IsExceptions = true; //kill wnd if exception doesn't match
									gg = false;
									QuitLooping = true;
								}
							}
							else {
								//is no -
								int sz = atoi(grej.String());
								if(size == sz) {
									IsExceptions = false; //don't kill wnd if excpeption match
									QuitLooping = true;
								} else {
									IsExceptions = true; //kill wnd if exception doesn't match
									QuitLooping = true;
								}
							} //b_error
						} //for
					} //b_error
				}
			}
		}
		else {
			QuitLooping = true;
		}
	}
	QuitLooping = false;
	//check textconditions
	if(IsExceptions) //if the window is about to be removed, check more carefully if that really is the case...
	{
	pos = 0;
	pos_end = 0;
	while(QuitLooping == false){
		pos = NaggerListContent.FindFirst("TextConditionException",pos_end);
		if(pos != B_ERROR) {
			pos_end = NaggerListContent.FindFirst('\n',pos);
			if(pos_end != B_ERROR) {
				if(pos>0){
					startcut = pos + strlen("TextConditionException")+1;
					endcut = pos_end - pos - strlen("TextConditionException");
					NaggerListContent.CopyInto(buffer,startcut,endcut);
					buffer.Truncate(buffer.CountChars()-2);
					if(strcmp(buffer.String(),window_title) == false) {
						IsExceptions = false; //don't kill
						QuitLooping = true;
					} else {
						if(strcmp(buffer.String(),"*") == false) {
							IsExceptions = false; //don't kill
							QuitLooping = true;
						} else {
							//check for wildcards
							IsExceptions = true; //kill
							QuitLooping = true;
						} //(any window)
					} //cond.=title?
				}
			}
		}
		else {
			QuitLooping = true;
		}
	}
	} //isexceptions?
	QuitLooping = false;
	
	//if there is exceptions:(false), if there no exceptions:(true)
	return IsExceptions;
}

status_t
NagApplication::ReadSettingsFile()
{
	//read file
	BFile *ConfFile = new BFile(NabberConfigFile,B_READ_ONLY);
	off_t storlek;
	ConfFile->GetSize(&storlek);
	char *stringData;
	stringData = (char *) malloc(storlek);
	ConfFile->Read((void *)stringData, storlek);
	
	//copy into BString
	fileContent.SetTo((const char *)stringData, strlen(stringData));

	return B_OK;
}


status_t
NagApplication::ReadNaggerList()
{
	//read file
	BFile *ConfFile = new BFile(NabberNaggerList,B_READ_ONLY);
	off_t storlek;
	ConfFile->GetSize(&storlek);
	char *stringData;
	stringData = (char *) malloc(storlek);
	ConfFile->Read((void *)stringData, storlek);
	
	//copy into BString
	NaggerListContent.SetTo((const char *)stringData, strlen(stringData));

	return B_OK;
}



